package com.ims.eval.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.PerformanceBenchmarkInfo;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IPerformanceBenchmarkInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * 业绩对标管理
 *
 * @author hlf
 * @date 2024/3/21 15:58
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-performance-benchmarking")
public class PerformanceBenchmarkingManageController {

	@Resource
	private IPerformanceBenchmarkInfoService performanceBenchmarkInfoService;

	/**
	 * 业绩对标管理列表信息（分页）
	 *
	 * @param pageNum        当前记录起始索引
	 * @param pageSize       每页显示记录数
	 * @param toolCategory   工具类别（寻标、立标、标杆、达标、超标）
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "toolCategory") String toolCategory,
		@RequestParam(value = "manageCategory") String manageCategory,
		@RequestParam(value = "checkCycle") String checkCycle) {
		IPage<PerformanceBenchmarkInfo> list = performanceBenchmarkInfoService.listPage(pageNum, pageSize, toolCategory, manageCategory, checkCycle);
		return R.ok().data(list);
	}

	/**
	 * 立标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @return 结果
	 */
	@PostMapping(value = "/lb")
	public R lb(
		@RequestParam(value = "manageCategory") String manageCategory,
		@RequestParam(value = "checkCycle") String checkCycle) {
		performanceBenchmarkInfoService.setRecommendedValue(manageCategory, checkCycle);
		return R.ok();
	}

	/**
	 * 标杆
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @return 结果
	 */
	@PostMapping(value = "/bg")
	public R bg(
		@RequestParam(value = "manageCategory") String manageCategory,
		@RequestParam(value = "checkCycle") String checkCycle) {
		performanceBenchmarkInfoService.bg(manageCategory, checkCycle);
		return R.ok();
	}

	/**
	 * 达标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @return 结果
	 */
	@PostMapping(value = "/db")
	public R db(
		@RequestParam(value = "manageCategory") String manageCategory,
		@RequestParam(value = "checkCycle") String checkCycle) {
		performanceBenchmarkInfoService.db(manageCategory, checkCycle);
		return R.ok();
	}

	/**
	 * 超标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @return 结果
	 */
	@PostMapping(value = "/cb")
	public R cb(
		@RequestParam(value = "manageCategory") String manageCategory,
		@RequestParam(value = "checkCycle") String checkCycle) {
		performanceBenchmarkInfoService.cb(manageCategory, checkCycle);
		return R.ok();
	}

	/**
	 * 修改业绩对标管理信息
	 *
	 * @param performanceBenchmarkInfo 业绩对标管理实体
	 * @return 结果
	 */
	@PostMapping(value = "/update")
	public R update(@RequestBody PerformanceBenchmarkInfo performanceBenchmarkInfo) {
		try {
			boolean b = performanceBenchmarkInfoService.updateById(performanceBenchmarkInfo);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("修改失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}
}
