package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.PerformanceBenchmarkInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.service.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author hlf
 * @date 2024/3/28 14:07
 * 文件说明：
 */
@Service
public class PerformanceBenchmarkInfoServiceImpl extends ServiceImpl<PerformanceBenchmarkInfoMapper, PerformanceBenchmarkInfo> implements IPerformanceBenchmarkInfoService {

	@Resource
	private IEvaluateRuleService evaluateRuleService;

	@Resource
	private IEvaluateRuleInfoService evaluateRuleInfoService;

	@Resource
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Resource
	private IOrganizationEvaluationInfoService organizationEvaluationInfoService;

	@Override
	public IPage<PerformanceBenchmarkInfo> listPage(Integer pageNum, Integer pageSize, String toolCategory, String manageCategory, String checkCycle) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<PerformanceBenchmarkInfo> page = new Page<>(pageNum, pageSize);
		int isBg = 0;
		int isDb = 0;
		int isCb = 0;
		if ("寻标".equals(toolCategory)) {
			xb(checkCycle, manageCategory);
		} else if ("标杆".equals(toolCategory)) {
			isBg = 1;
		} else if ("达标".equals(toolCategory)) {
			isDb = 2;
		} else if ("超标".equals(toolCategory)) {
			isCb = 3;
		}
		return baseMapper.selectListPage(page, manageCategory, checkCycle, isBg, isDb, isCb);
	}

	/**
	 * 寻标
	 */
	private void xb(String checkCycle, String manageCategory) {
		List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = new ArrayList<>();
		EvaluateRule evaluateRule = evaluateRuleService.getEvaluateRuleListByYear("", checkCycle, manageCategory);
		if (null != evaluateRule) {
			List<EvaluateRuleInfo> evaluateRuleInfoList = evaluateRuleInfoService.getEvaluateRuleInfoByEvaluateRuleId(evaluateRule.getId());
			for (EvaluateRuleInfo evaluateRuleInfo : evaluateRuleInfoList) {
				List<IndicatorDictionary> indicatorDictionaryList = indicatorDictionaryService.getIndicatorDictionaryList(evaluateRuleInfo.getIndicatorId());
				if (indicatorDictionaryList.size() > 0) {
					//计划值
					IndicatorDictionary jhz = indicatorDictionaryList.stream().filter(p -> !p.getOptionCode().equals("WCZ")).findFirst().orElse(null);
					//实际完成值
					IndicatorDictionary wcz = indicatorDictionaryList.stream().filter(p -> p.getOptionCode().equals("WCZ")).findFirst().orElse(null);
					if (jhz != null && wcz != null) {
						List<OrganizationEvaluationInfo> jhzList = organizationEvaluationInfoService.getOrganizationEvaluationInfoListByIndicatorDictionaryId(jhz.getId());
						List<OrganizationEvaluationInfo> wczList = organizationEvaluationInfoService.getOrganizationEvaluationInfoListByIndicatorDictionaryId(wcz.getId());
						if (jhzList.size() == wczList.size()) {
							for (int i = 0; i < wczList.size(); i++) {
								OrganizationEvaluationInfo jhzObj = jhzList.get(i);
								OrganizationEvaluationInfo wczObj = wczList.get(i);
								PerformanceBenchmarkInfo performanceBenchmarkInfo = new PerformanceBenchmarkInfo();
								performanceBenchmarkInfo.setId(Convert.toStr(IdUtil.getSnowflake(1, 1).nextId()));
								performanceBenchmarkInfo.setCheckCycle(wczObj.getCheckCycle());
								performanceBenchmarkInfo.setDegreeYear(Integer.valueOf(wczObj.getDegreeYear()));
								if (!"NDKP".equals(checkCycle)) {
									performanceBenchmarkInfo.setQuarterlyMonth(Integer.valueOf(wczObj.getQuarterlyMonth()));
								}
								performanceBenchmarkInfo.setManageCategory(manageCategory);
								performanceBenchmarkInfo.setCompanyId(wczObj.getOrganizationId());
								performanceBenchmarkInfo.setCompanyName(wczObj.getCompanyName());
								performanceBenchmarkInfo.setCompanyShort(wczObj.getCompanyShort());
								performanceBenchmarkInfo.setInstallCapacity(wczObj.getInstallCapacity());
								performanceBenchmarkInfo.setIndexName(evaluateRuleInfo.getIndicatorName());
								performanceBenchmarkInfo.setPlannedValue(jhzObj.getQuantifiedValue());
								performanceBenchmarkInfo.setCompleteValue(wczObj.getQuantifiedValue());
								performanceBenchmarkInfo.setIsBg(0);
								performanceBenchmarkInfo.setIsDb(0);
								performanceBenchmarkInfo.setIsCb(0);
								if (uniquenessCheck(performanceBenchmarkInfo)) {
									performanceBenchmarkInfoList.add(performanceBenchmarkInfo);
								}
							}
						}
					}
				}
			}
		}
		super.saveBatch(performanceBenchmarkInfoList);
	}

	@Override
	public void setRecommendedValue(String manageCategory, String checkCycle) {
		QueryWrapper<PerformanceBenchmarkInfo> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(manageCategory)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getManageCategory, manageCategory);
		}
		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getCheckCycle, checkCycle);
		}
		if ("NDKP".equals(checkCycle)) {
			Map<Integer, Map<String, List<PerformanceBenchmarkInfo>>> grouped = baseMapper.selectList(qw).stream()
				.collect(Collectors.groupingBy(
					PerformanceBenchmarkInfo::getDegreeYear,
					Collectors.groupingBy(
						PerformanceBenchmarkInfo::getIndexName
					)
				));
			List<PerformanceBenchmarkInfo> updateList = new ArrayList<>();
			grouped.forEach((degreeYear, indexNameMap) -> {
				indexNameMap.forEach((indexName, performanceBenchmarkInfoList) -> {
					double completeValueSum = performanceBenchmarkInfoList.stream().mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).sum();
					double installCapacitySum = performanceBenchmarkInfoList.stream().mapToDouble(PerformanceBenchmarkInfo::getInstallCapacity).sum();
					performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {
						PerformanceBenchmarkInfo.setRecommendedValue(yearIndexRecommendedValueA(completeValueSum, installCapacitySum, PerformanceBenchmarkInfo.getInstallCapacity()));
						updateList.add(PerformanceBenchmarkInfo);
					});
				});
			});
			super.updateBatchById(updateList);
		} else if ("JDKP".equals(checkCycle) || "YDKP".equals(checkCycle)) {
			Map<Integer, Map<Integer, Map<String, List<PerformanceBenchmarkInfo>>>> grouped = baseMapper.selectList(qw).stream()
				.collect(Collectors.groupingBy(
					PerformanceBenchmarkInfo::getDegreeYear,
					Collectors.groupingBy(
						PerformanceBenchmarkInfo::getQuarterlyMonth,
						Collectors.groupingBy(
							PerformanceBenchmarkInfo::getIndexName,
							Collectors.toList()
						)
					)
				));
			List<PerformanceBenchmarkInfo> updateList = new ArrayList<>();
			grouped.forEach((degreeYear, quarterlyMonthMap) -> {
				quarterlyMonthMap.forEach((quarterlyMonth, indexNameMap) -> {
					indexNameMap.forEach((indexName, performanceBenchmarkInfoList) -> {
						performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {

						});
					});
				});
			});
		}
	}

	@Override
	public void bg(String manageCategory, String checkCycle) {
		QueryWrapper<PerformanceBenchmarkInfo> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(manageCategory)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getManageCategory, manageCategory);
		}
		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getCheckCycle, checkCycle);
		}
		Map<Integer, Map<String, List<PerformanceBenchmarkInfo>>> grouped = baseMapper.selectList(qw).stream()
			.collect(Collectors.groupingBy(
				PerformanceBenchmarkInfo::getDegreeYear,
				Collectors.groupingBy(
					PerformanceBenchmarkInfo::getIndexName
				)
			));
		grouped.forEach((degreeYear, indexNameMap) -> {
			indexNameMap.forEach((indexName, performanceBenchmarkInfoList) -> {
				performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {
					PerformanceBenchmarkInfo.setBgVal(PerformanceBenchmarkInfo.getInstallCapacity() / PerformanceBenchmarkInfo.getCompleteValue());
				});
				performanceBenchmarkInfoList.sort(Comparator.comparingDouble(PerformanceBenchmarkInfo::getBgVal).reversed());
				if (!performanceBenchmarkInfoList.isEmpty() && performanceBenchmarkInfoList.get(0).getBgVal() != 0) {
					performanceBenchmarkInfoList.get(0).setIsBg(1);
				}
				super.updateBatchById(performanceBenchmarkInfoList);
			});
		});
	}

	@Override
	public void db(String manageCategory, String checkCycle) {
		QueryWrapper<PerformanceBenchmarkInfo> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(manageCategory)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getManageCategory, manageCategory);
		}
		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getCheckCycle, checkCycle);
		}
		List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(qw);
		for (PerformanceBenchmarkInfo performanceBenchmarkInfo : performanceBenchmarkInfoList) {
			if (performanceBenchmarkInfo.getCompleteValue() >= performanceBenchmarkInfo.getScalingValue()) {
				performanceBenchmarkInfo.setIsDb(1);
				performanceBenchmarkInfo.setDbVal(performanceBenchmarkInfo.getCompleteValue() - performanceBenchmarkInfo.getScalingValue());
			}
		}
		super.updateBatchById(performanceBenchmarkInfoList);
	}

	@Override
	public void cb(String manageCategory, String checkCycle) {
		QueryWrapper<PerformanceBenchmarkInfo> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(manageCategory)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getManageCategory, manageCategory);
		}
		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getCheckCycle, checkCycle);
		}
		Map<Integer, Map<String, List<PerformanceBenchmarkInfo>>> grouped = baseMapper.selectList(qw).stream()
			.collect(Collectors.groupingBy(
				PerformanceBenchmarkInfo::getDegreeYear,
				Collectors.groupingBy(
					PerformanceBenchmarkInfo::getIndexName
				)
			));
		grouped.forEach((degreeYear, indexNameMap) -> {
			indexNameMap.forEach((indexName, performanceBenchmarkInfoList) -> {
				performanceBenchmarkInfoList.stream().filter(p -> p.getIsBg().equals(1)).findFirst().ifPresent(performanceBenchmarkInfo -> performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {
					if (PerformanceBenchmarkInfo.getCompleteValue() > performanceBenchmarkInfo.getScalingValue()) {
						PerformanceBenchmarkInfo.setIsCb(1);
						PerformanceBenchmarkInfo.setCbVal(PerformanceBenchmarkInfo.getCompleteValue() - performanceBenchmarkInfo.getScalingValue());
					}
				}));
				super.updateBatchById(performanceBenchmarkInfoList);
			});
		});
	}

	/**
	 * 年度指标推荐值A
	 */
	private Double yearIndexRecommendedValueA(double a, double b, double c) {
		return (a / b) * c;
	}

	/**
	 * 年度指标推荐值B
	 */
	private Double yearIndexRecommendedValueB(double a, double b, double c) {
		double B;
		if (b == 0) {
			B = a * 0.02;
		} else {
			B = (a / b) * c;
		}
		return B;
	}

	/**
	 * 季/月度指标推荐值
	 */
	private Double quarterlyMonthIndexRecommendedValue(double a, double b) {
		return a * b;
	}

	private boolean uniquenessCheck(PerformanceBenchmarkInfo performanceBenchmarkInfo) {
		QueryWrapper<PerformanceBenchmarkInfo> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(performanceBenchmarkInfo.getCheckCycle())) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getCheckCycle, performanceBenchmarkInfo.getCheckCycle());
		}
		if (null != performanceBenchmarkInfo.getDegreeYear()) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getDegreeYear, performanceBenchmarkInfo.getDegreeYear());
		}
		if (null != performanceBenchmarkInfo.getQuarterlyMonth()) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getQuarterlyMonth, performanceBenchmarkInfo.getQuarterlyMonth());
		}
		if (StringUtils.isNotEmpty(performanceBenchmarkInfo.getManageCategory())) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getManageCategory, performanceBenchmarkInfo.getManageCategory());
		}
		if (StringUtils.isNotEmpty(performanceBenchmarkInfo.getCompanyId())) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getCompanyId, performanceBenchmarkInfo.getCompanyId());
		}
		if (null != performanceBenchmarkInfo.getPlannedValue()) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getPlannedValue, performanceBenchmarkInfo.getPlannedValue());
		}
		if (null != performanceBenchmarkInfo.getCompleteValue()) {
			qw.lambda().eq(PerformanceBenchmarkInfo::getCompleteValue, performanceBenchmarkInfo.getCompleteValue());
		}
		List<PerformanceBenchmarkInfo> objList = baseMapper.selectList(qw);
		return objList.size() == 0;
	}
}
